@@ -3,14 +3,11 @@ package color
 import (
 	"fmt"
 	"oh-my-posh/regex"
+	"oh-my-posh/shell"
 	"strings"
 )
 
 const (
-	zsh  = "zsh"
-	bash = "bash"
-	pwsh = "pwsh"
-
 	AnsiRegex = "[\u001B\u009B][[\\]()#;?]*(?:(?:(?:[a-zA-Z\\d]*(?:;[a-zA-Z\\d]*)*)?\u0007)|(?:(?:\\d{1,4}(?:;\\d{0,4})*)?[\\dA-PRZcf-ntqry=><~]))"
 )
 
@@ -31,6 +28,7 @@ type Ansi struct {
 	escapeLeft            string
 	escapeRight           string
 	hyperlink             string
+	hyperlinkRegex        string
 	osc99                 string
 	bold                  string
 	italic                string
@@ -40,18 +38,19 @@ type Ansi struct {
 	reverse               string
 	dimmed                string
 	format                string
-	shellReservedKeywords []shellKeyWordReplacement
+	reservedSequences     []sequenceReplacement
 }
 
-type shellKeyWordReplacement struct {
+type sequenceReplacement struct {
 	text        string
 	replacement string
 }
 
-func (a *Ansi) Init(shell string) {
-	a.shell = shell
-	switch shell {
-	case zsh:
+func (a *Ansi) Init(shellName string) {
+	a.shell = shellName
+	a.initEscapeSequences(shellName)
+	switch shellName {
+	case shell.ZSH:
 		a.format = "%%{%s%%}"
 		a.linechange = "%%{\x1b[%d%s%%}"
 		a.right = "%%{\x1b[%dC%%}"
@@ -68,6 +67,7 @@ func (a *Ansi) Init(shell string) {
 		a.escapeLeft = "%{"
 		a.escapeRight = "%}"
 		a.hyperlink = "%%{\x1b]8;;%s\x1b\\%%}%s%%{\x1b]8;;\x1b\\%%}"
+		a.hyperlinkRegex = `(?P<STR>%{\x1b]8;;(.+)\x1b\\%}(?P<TEXT>.+)%{\x1b]8;;\x1b\\%})`
 		a.osc99 = "%%{\x1b]9;9;\"%s\"\x1b\\%%}"
 		a.bold = "%%{\x1b[1m%%}%s%%{\x1b[22m%%}"
 		a.italic = "%%{\x1b[3m%%}%s%%{\x1b[23m%%}"
@@ -76,9 +76,7 @@ func (a *Ansi) Init(shell string) {
 		a.reverse = "%%{\x1b[7m%%}%s%%{\x1b[27m%%}"
 		a.dimmed = "%%{\x1b[2m%%}%s%%{\x1b[22m%%}"
 		a.strikethrough = "%%{\x1b[9m%%}%s%%{\x1b[29m%%}"
-		// escape double quotes and variable expansion
-		a.shellReservedKeywords = append(a.shellReservedKeywords, shellKeyWordReplacement{"\\", "\\\\"}, shellKeyWordReplacement{"%", "%%"})
-	case bash:
+	case shell.BASH:
 		a.format = "\\[%s\\]"
 		a.linechange = "\\[\x1b[%d%s\\]"
 		a.right = "\\[\x1b[%dC\\]"
@@ -95,17 +93,15 @@ func (a *Ansi) Init(shell string) {
 		a.escapeLeft = "\\["
 		a.escapeRight = "\\]"
 		a.hyperlink = "\\[\x1b]8;;%s\x1b\\\\\\]%s\\[\x1b]8;;\x1b\\\\\\]"
+		a.hyperlinkRegex = `(?P<STR>\\\[\x1b\]8;;(.+)\x1b\\\\\\\](?P<TEXT>.+)\\\[\x1b\]8;;\x1b\\\\\\\])`
 		a.osc99 = "\\[\x1b]9;9;\"%s\"\x1b\\\\\\]"
 		a.bold = "\\[\x1b[1m\\]%s\\[\x1b[22m\\]"
 		a.italic = "\\[\x1b[3m\\]%s\\[\x1b[23m\\]"
 		a.underline = "\\[\x1b[4m\\]%s\\[\x1b[24m\\]"
-		a.blink = "\\[\x1b[5m%s\\[\x1b[25m\\]"
+		a.blink = "\\[\x1b[5m\\]%s\\[\x1b[25m\\]"
 		a.reverse = "\\[\x1b[7m\\]%s\\[\x1b[27m\\]"
 		a.dimmed = "\\[\x1b[2m\\]%s\\[\x1b[22m\\]"
 		a.strikethrough = "\\[\x1b[9m\\]%s\\[\x1b[29m\\]"
-		// escape backslashes to avoid replacements
-		// https://tldp.org/HOWTO/Bash-Prompt-HOWTO/bash-prompt-escape-sequences.html
-		a.shellReservedKeywords = append(a.shellReservedKeywords, shellKeyWordReplacement{"\\", "\\\\"})
 	default:
 		a.format = "%s"
 		a.linechange = "\x1b[%d%s"
@@ -123,6 +119,7 @@ func (a *Ansi) Init(shell string) {
 		a.escapeLeft = ""
 		a.escapeRight = ""
 		a.hyperlink = "\x1b]8;;%s\x1b\\%s\x1b]8;;\x1b\\"
+		a.hyperlinkRegex = "(?P<STR>\x1b]8;;(.+)\x1b\\\\\\\\?(?P<TEXT>.+)\x1b]8;;\x1b\\\\)"
 		a.osc99 = "\x1b]9;9;\"%s\"\x1b\\"
 		a.bold = "\x1b[1m%s\x1b[22m"
 		a.italic = "\x1b[3m%s\x1b[23m"
@@ -132,11 +129,109 @@ func (a *Ansi) Init(shell string) {
 		a.dimmed = "\x1b[2m%s\x1b[22m"
 		a.strikethrough = "\x1b[9m%s\x1b[29m"
 	}
-	// common replacement for all shells
-	a.shellReservedKeywords = append(a.shellReservedKeywords, shellKeyWordReplacement{"`", "'"})
 }
 
-func (a *Ansi) generateHyperlink(text string) string {
+func (a *Ansi) InitPlain(shellName string) {
+	a.Init(shell.PLAIN)
+	a.initEscapeSequences(shellName)
+}
+
+func (a *Ansi) initEscapeSequences(shellName string) {
+	switch shellName {
+	case shell.ZSH:
+		// escape double quotes and variable expansion
+		a.reservedSequences = []sequenceReplacement{
+			{text: "`", replacement: "'"},
+			{text: `%l`, replacement: `%%l`},
+			{text: `%M`, replacement: `%%M`},
+			{text: `%m`, replacement: `%%m`},
+			{text: `%n`, replacement: `%%n`},
+			{text: `%y`, replacement: `%%y`},
+			{text: `%#`, replacement: `%%#`},
+			{text: `%?`, replacement: `%%?`},
+			{text: `%_`, replacement: `%%_`},
+			{text: `%^`, replacement: `%%^`},
+			{text: `%d`, replacement: `%%d`},
+			{text: `%/`, replacement: `%%/`},
+			{text: `%~`, replacement: `%%~`},
+			{text: `%e`, replacement: `%%e`},
+			{text: `%h`, replacement: `%%h`},
+			{text: `%!`, replacement: `%%!`},
+			{text: `%i`, replacement: `%%i`},
+			{text: `%I`, replacement: `%%I`},
+			{text: `%j`, replacement: `%%j`},
+			{text: `%L`, replacement: `%%L`},
+			{text: `%N`, replacement: `%%N`},
+			{text: `%x`, replacement: `%%x`},
+			{text: `%c`, replacement: `%%c`},
+			{text: `%.`, replacement: `%%.`},
+			{text: `%C`, replacement: `%%C`},
+			{text: `%D`, replacement: `%%D`},
+			{text: `%T`, replacement: `%%T`},
+			{text: `%t`, replacement: `%%t`},
+			{text: `%@`, replacement: `%%@`},
+			{text: `%*`, replacement: `%%*`},
+			{text: `%w`, replacement: `%%w`},
+			{text: `%W`, replacement: `%%W`},
+			{text: `%D`, replacement: `%%D`},
+			{text: `%B`, replacement: `%%B`},
+			{text: `%b`, replacement: `%%b`},
+			{text: `%E`, replacement: `%%E`},
+			{text: `%U`, replacement: `%%U`},
+			{text: `%S`, replacement: `%%S`},
+			{text: `%F`, replacement: `%%F`},
+			{text: `%K`, replacement: `%%K`},
+			{text: `%G`, replacement: `%%G`},
+			{text: `%v`, replacement: `%%v`},
+			{text: `%(`, replacement: `%%(`},
+		}
+	case shell.BASH:
+		a.reservedSequences = []sequenceReplacement{
+			{text: "`", replacement: "'"},
+			{text: `\a`, replacement: `\\a`},
+			{text: `\d`, replacement: `\\d`},
+			{text: `\D`, replacement: `\\D`},
+			{text: `\e`, replacement: `\\e`},
+			{text: `\h`, replacement: `\\h`},
+			{text: `\H`, replacement: `\\H`},
+			{text: `\j`, replacement: `\\j`},
+			{text: `\l`, replacement: `\\l`},
+			{text: `\n`, replacement: `\\n`},
+			{text: `\r`, replacement: `\\r`},
+			{text: `\s`, replacement: `\\s`},
+			{text: `\t`, replacement: `\\t`},
+			{text: `\T`, replacement: `\\T`},
+			{text: `\@`, replacement: `\\@`},
+			{text: `\A`, replacement: `\\A`},
+			{text: `\u`, replacement: `\\u`},
+			{text: `\v`, replacement: `\\v`},
+			{text: `\V`, replacement: `\\V`},
+			{text: `\w`, replacement: `\\w`},
+			{text: `\W`, replacement: `\\W`},
+			{text: `\!`, replacement: `\\!`},
+			{text: `\#`, replacement: `\\#`},
+			{text: `\$`, replacement: `\\$`},
+			{text: `\nnn`, replacement: `\\nnn`},
+		}
+	case shell.FISH:
+		a.reservedSequences = []sequenceReplacement{
+			{text: "`", replacement: "'"},
+			{text: `\a`, replacement: `\\a`},
+			{text: `\b`, replacement: `\\b`},
+			{text: `\e`, replacement: `\\e`},
+			{text: `\f`, replacement: `\\f`},
+			{text: `\r`, replacement: `\\r`},
+			{text: `\t`, replacement: `\\t`},
+			{text: `\v`, replacement: `\\v`},
+		}
+	default:
+		a.reservedSequences = []sequenceReplacement{
+			{text: "`", replacement: "'"},
+		}
+	}
+}
+
+func (a *Ansi) GenerateHyperlink(text string) string {
 	// hyperlink matching
 	results := regex.FindNamedRegexMatch("(?P<all>(?:\\[(?P<name>.+)\\])(?:\\((?P<url>.*)\\)))", text)
 	if len(results) != 3 {
@@ -209,7 +304,8 @@ func (a *Ansi) ClearAfter() string {
 
 func (a *Ansi) EscapeText(text string) string {
 	// what to escape/replace is different per shell
-	for _, s := range a.shellReservedKeywords {
+
+	for _, s := range a.reservedSequences {
 		text = strings.ReplaceAll(text, s.text, s.replacement)
 	}
 	return text
