@@ -3,6 +3,7 @@ package engine
 import (
 	"errors"
 	"fmt"
+	"oh-my-posh/color"
 	"oh-my-posh/environment"
 	"oh-my-posh/properties"
 	"oh-my-posh/segments"
@@ -14,18 +15,19 @@ import (
 
 // Segment represent a single segment and it's configuration
 type Segment struct {
-	Type                SegmentType    `json:"type,omitempty"`
-	Tips                []string       `json:"tips,omitempty"`
-	Style               SegmentStyle   `json:"style,omitempty"`
-	PowerlineSymbol     string         `json:"powerline_symbol,omitempty"`
-	InvertPowerline     bool           `json:"invert_powerline,omitempty"`
-	Foreground          string         `json:"foreground,omitempty"`
-	ForegroundTemplates []string       `json:"foreground_templates,omitempty"`
-	Background          string         `json:"background,omitempty"`
-	BackgroundTemplates []string       `json:"background_templates,omitempty"`
-	LeadingDiamond      string         `json:"leading_diamond,omitempty"`
-	TrailingDiamond     string         `json:"trailing_diamond,omitempty"`
-	Properties          properties.Map `json:"properties,omitempty"`
+	Type                SegmentType     `json:"type,omitempty"`
+	Tips                []string        `json:"tips,omitempty"`
+	Style               SegmentStyle    `json:"style,omitempty"`
+	PowerlineSymbol     string          `json:"powerline_symbol,omitempty"`
+	InvertPowerline     bool            `json:"invert_powerline,omitempty"`
+	Foreground          string          `json:"foreground,omitempty"`
+	ForegroundTemplates color.Templates `json:"foreground_templates,omitempty"`
+	Background          string          `json:"background,omitempty"`
+	BackgroundTemplates color.Templates `json:"background_templates,omitempty"`
+	LeadingDiamond      string          `json:"leading_diamond,omitempty"`
+	TrailingDiamond     string          `json:"trailing_diamond,omitempty"`
+	Template            string          `json:"template,omitempty"`
+	Properties          properties.Map  `json:"properties,omitempty"`
 
 	writer          SegmentWriter
 	text            string
@@ -58,6 +60,15 @@ type SegmentStyle string
 type SegmentType string
 
 const (
+	// Powerline writes it Powerline style
+	Powerline SegmentStyle = "powerline"
+	// Accordion writes it Powerline style but collapses the segment when disabled instead of hiding
+	Accordion SegmentStyle = "accordion"
+	// Plain writes it without ornaments
+	Plain SegmentStyle = "plain"
+	// Diamond writes the prompt shaped with a leading and trailing symbol
+	Diamond SegmentStyle = "diamond"
+
 	// SESSION represents the user info segment
 	SESSION SegmentType = "session"
 	// PATH represents the current path segment
@@ -100,12 +111,6 @@ const (
 	GOLANG SegmentType = "go"
 	// JULIA writes which julia version is currently active
 	JULIA SegmentType = "julia"
-	// Powerline writes it Powerline style
-	Powerline SegmentStyle = "powerline"
-	// Plain writes it without ornaments
-	Plain SegmentStyle = "plain"
-	// Diamond writes the prompt shaped with a leading and trailing symbol
-	Diamond SegmentStyle = "diamond"
 	// YTM writes YouTube Music information and status
 	YTM SegmentType = "ytm"
 	// EXECUTIONTIME writes the execution time of the last run command
@@ -162,14 +167,29 @@ const (
 	KOTLIN SegmentType = "kotlin"
 	// SWIFT writes the active swift version
 	SWIFT SegmentType = "swift"
+	// cds (SAP CAP) version
+	CDS SegmentType = "cds"
+	// npm version
+	NPM SegmentType = "npm"
+	// Project version
+	PROJECT SegmentType = "project"
+	// R version
+	R SegmentType = "r"
 )
 
 func (segment *Segment) shouldIncludeFolder() bool {
+	if segment.env == nil {
+		return true
+	}
 	cwdIncluded := segment.cwdIncluded()
 	cwdExcluded := segment.cwdExcluded()
 	return cwdIncluded && !cwdExcluded
 }
 
+func (segment *Segment) isPowerline() bool {
+	return segment.Style == Powerline || segment.Style == Accordion
+}
+
 func (segment *Segment) cwdIncluded() bool {
 	value, ok := segment.Properties[properties.IncludeFolders]
 	if !ok {
@@ -184,7 +204,7 @@ func (segment *Segment) cwdIncluded() bool {
 		return true
 	}
 
-	return environment.DirMatchesOneOf(segment.env, segment.env.Pwd(), list)
+	return segment.env.DirMatchesOneOf(segment.env.Pwd(), list)
 }
 
 func (segment *Segment) cwdExcluded() bool {
@@ -193,26 +213,7 @@ func (segment *Segment) cwdExcluded() bool {
 		value = segment.Properties[properties.IgnoreFolders]
 	}
 	list := properties.ParseStringArray(value)
-	return environment.DirMatchesOneOf(segment.env, segment.env.Pwd(), list)
-}
-
-func (segment *Segment) getColor(templates []string, defaultColor string) string {
-	if len(templates) == 0 {
-		return defaultColor
-	}
-	txtTemplate := &template.Text{
-		Context: segment.writer,
-		Env:     segment.env,
-	}
-	for _, tmpl := range templates {
-		txtTemplate.Template = tmpl
-		value, err := txtTemplate.Render()
-		if err != nil || value == "" {
-			continue
-		}
-		return value
-	}
-	return defaultColor
+	return segment.env.DirMatchesOneOf(segment.env.Pwd(), list)
 }
 
 func (segment *Segment) shouldInvokeWithTip(tip string) bool {
@@ -226,14 +227,14 @@ func (segment *Segment) shouldInvokeWithTip(tip string) bool {
 
 func (segment *Segment) foreground() string {
 	if len(segment.foregroundCache) == 0 {
-		segment.foregroundCache = segment.getColor(segment.ForegroundTemplates, segment.Foreground)
+		segment.foregroundCache = segment.ForegroundTemplates.Resolve(segment.writer, segment.env, segment.Foreground)
 	}
 	return segment.foregroundCache
 }
 
 func (segment *Segment) background() string {
 	if len(segment.backgroundCache) == 0 {
-		segment.backgroundCache = segment.getColor(segment.BackgroundTemplates, segment.Background)
+		segment.backgroundCache = segment.BackgroundTemplates.Resolve(segment.writer, segment.env, segment.Background)
 	}
 	return segment.backgroundCache
 }
@@ -241,6 +242,8 @@ func (segment *Segment) background() string {
 func (segment *Segment) mapSegmentWithWriter(env environment.Environment) error {
 	segment.env = env
 	functions := map[SegmentType]SegmentWriter{
+		PROJECT:       &segments.Project{},
+		NPM:           &segments.Npm{},
 		OWM:           &segments.Owm{},
 		SESSION:       &segments.Session{},
 		PATH:          &segments.Path{},
@@ -290,6 +293,8 @@ func (segment *Segment) mapSegmentWithWriter(env environment.Environment) error
 		CFTARGET:      &segments.CfTarget{},
 		KOTLIN:        &segments.Kotlin{},
 		SWIFT:         &segments.Swift{},
+		CDS:           &segments.Cds{},
+		R:             &segments.R{},
 	}
 	if segment.Properties == nil {
 		segment.Properties = make(properties.Map)
@@ -303,9 +308,11 @@ func (segment *Segment) mapSegmentWithWriter(env environment.Environment) error
 }
 
 func (segment *Segment) string() string {
-	segmentTemplate := segment.Properties.GetString(properties.SegmentTemplate, segment.writer.Template())
+	if len(segment.Template) == 0 {
+		segment.Template = segment.writer.Template()
+	}
 	tmpl := &template.Text{
-		Template: segmentTemplate,
+		Template: segment.Template,
 		Context:  segment.writer,
 		Env:      segment.env,
 	}
