@@ -6,16 +6,18 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"io/fs"
 	"io/ioutil"
 	"log"
 	"net/http"
-	"net/url"
 	"oh-my-posh/regex"
 	"os"
 	"os/exec"
 	"path/filepath"
 	"runtime"
+	"strconv"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/distatus/battery"
@@ -29,39 +31,17 @@ const (
 	LinuxPlatform   = "linux"
 )
 
-type Args struct {
-	ErrorCode      *int
-	PrintInit      *bool
-	PrintConfig    *bool
-	PrintShell     *bool
-	PrintTransient *bool
-	PrintSecondary *bool
-	PrintValid     *bool
-	PrintError     *bool
-	Config         *string
-	ConfigFormat   *string
-	Shell          *string
-	PWD            *string
-	PSWD           *string
-	Version        *bool
-	Debug          *bool
-	ExecutionTime  *float64
-	Millis         *bool
-	Eval           *bool
-	Init           *bool
-	ExportPNG      *bool
-	Author         *string
-	CursorPadding  *int
-	RPromptOffset  *int
-	RPrompt        *bool
-	BGColor        *string
-	StackCount     *int
-	Command        *string
-	Plain          *bool
-	CachePath      *bool
-	Migrate        *bool
-	Write          *bool
-	TerminalWidth  *int
+type Flags struct {
+	ErrorCode     int
+	Config        string
+	Shell         string
+	PWD           string
+	PSWD          string
+	ExecutionTime float64
+	Eval          bool
+	StackCount    int
+	Migrate       bool
+	TerminalWidth int
 }
 
 type CommandError struct {
@@ -156,15 +136,16 @@ type Environment interface {
 	HasFolder(folder string) bool
 	HasParentFilePath(path string) (fileInfo *FileInfo, err error)
 	HasFileInParentDirs(pattern string, depth uint) bool
+	DirMatchesOneOf(dir string, regexes []string) bool
 	HasCommand(command string) bool
 	FileContent(file string) string
-	FolderList(path string) []string
+	LsDir(path string) []fs.DirEntry
 	RunCommand(command string, args ...string) (string, error)
 	RunShellCommand(shell, command string) string
 	ExecutionTime() float64
-	Args() *Args
+	Flags() *Flags
 	BatteryInfo() ([]*battery.Battery, error)
-	WindowTitle(imageName, windowTitleRegex string) (string, error)
+	QueryWindowTitles(processName, windowTitleRegex string) (string, error)
 	WindowsRegistryKeyValue(path string) (*WindowsRegistryValue, error)
 	HTTPRequest(url string, timeout int, requestModifiers ...HTTPRequestModifier) ([]byte, error)
 	IsWsl() bool
@@ -207,62 +188,51 @@ const (
 )
 
 type ShellEnvironment struct {
-	args       *Args
+	CmdFlags *Flags
+	Version  string
+
 	cwd        string
 	cmdCache   *commandCache
 	fileCache  *fileCache
 	tmplCache  *TemplateCache
 	logBuilder strings.Builder
 	debug      bool
+	lock       sync.Mutex
 }
 
-func (env *ShellEnvironment) Init(args *Args) {
-	env.args = args
+func (env *ShellEnvironment) Init(debug bool) {
+	if env.CmdFlags == nil {
+		env.CmdFlags = &Flags{}
+	}
 	env.fileCache = &fileCache{}
 	env.fileCache.Init(env.CachePath())
-	env.ResolveConfigPath()
+	env.resolveConfigPath()
 	env.cmdCache = &commandCache{
 		commands: newConcurrentMap(),
 	}
-	if env.args != nil && *env.args.Debug {
+	if debug {
 		env.debug = true
 		log.SetOutput(&env.logBuilder)
 	}
 }
 
-func (env *ShellEnvironment) getConfigPath(location string) {
-	cfg, err := env.HTTPRequest(location, 5000)
-	if err != nil {
-		return
+func (env *ShellEnvironment) resolveConfigPath() {
+	if len(env.CmdFlags.Config) == 0 {
+		env.CmdFlags.Config = env.Getenv("POSH_THEME")
 	}
-	configPath := filepath.Join(env.CachePath(), "config.omp.json")
-	out, err := os.Create(configPath)
-	if err != nil {
-		return
+	if len(env.CmdFlags.Config) == 0 {
+		env.CmdFlags.Config = fmt.Sprintf("https://raw.githubusercontent.com/JanDeDobbeleer/oh-my-posh/v%s/themes/default.omp.json", env.Version)
 	}
-	defer out.Close()
-	_, err = io.Copy(out, bytes.NewReader(cfg))
-	if err != nil {
-		return
-	}
-	env.args.Config = &configPath
-}
-
-func (env *ShellEnvironment) ResolveConfigPath() {
-	if env.args == nil || env.args.Config == nil || len(*env.args.Config) == 0 {
-		return
-	}
-	location, err := url.ParseRequestURI(*env.Args().Config)
-	if err == nil {
-		env.getConfigPath(location.String())
+	if strings.HasPrefix(env.CmdFlags.Config, "https://") {
+		env.getConfigPath(env.CmdFlags.Config)
 		return
 	}
 	// Cygwin path always needs the full path as we're on Windows but not really.
 	// Doing filepath actions will convert it to a Windows path and break the init script.
-	if env.Platform() == WindowsPlatform && env.Shell() == "bash" {
+	if env.Platform() == WindowsPlatform && env.Shell() == "constants.BASH" {
 		return
 	}
-	configFile := *env.args.Config
+	configFile := env.CmdFlags.Config
 	if strings.HasPrefix(configFile, "~") {
 		configFile = strings.TrimPrefix(configFile, "~")
 		configFile = filepath.Join(env.Home(), configFile)
@@ -272,7 +242,44 @@ func (env *ShellEnvironment) ResolveConfigPath() {
 			configFile = absConfigFile
 		}
 	}
-	*env.args.Config = filepath.Clean(configFile)
+	env.CmdFlags.Config = filepath.Clean(configFile)
+}
+
+func (env *ShellEnvironment) getConfigPath(location string) {
+	configFileName := fmt.Sprintf("%s.omp.json", env.Version)
+	configPath := filepath.Join(env.CachePath(), configFileName)
+	if env.HasFilesInDir(env.CachePath(), configFileName) {
+		env.CmdFlags.Config = configPath
+		return
+	}
+	// clean old config files
+	cleanCacheDir := func() {
+		dir, err := ioutil.ReadDir(env.CachePath())
+		if err != nil {
+			return
+		}
+		for _, file := range dir {
+			if strings.HasSuffix(file.Name(), ".omp.json") {
+				os.Remove(filepath.Join(env.CachePath(), file.Name()))
+			}
+		}
+	}
+	cleanCacheDir()
+
+	cfg, err := env.HTTPRequest(location, 5000)
+	if err != nil {
+		return
+	}
+	out, err := os.Create(configPath)
+	if err != nil {
+		return
+	}
+	defer out.Close()
+	_, err = io.Copy(out, bytes.NewReader(cfg))
+	if err != nil {
+		return
+	}
+	env.CmdFlags.Config = configPath
 }
 
 func (env *ShellEnvironment) trace(start time.Time, function string, args ...string) {
@@ -292,6 +299,14 @@ func (env *ShellEnvironment) log(lt logType, function, message string) {
 	log.Println(trace)
 }
 
+func (env *ShellEnvironment) debugF(function string, fn func() string) {
+	if !env.debug {
+		return
+	}
+	trace := fmt.Sprintf("%s: %s\n%s", Debug, function, fn())
+	log.Println(trace)
+}
+
 func (env *ShellEnvironment) Getenv(key string) string {
 	defer env.trace(time.Now(), "Getenv", key)
 	val := os.Getenv(key)
@@ -301,7 +316,9 @@ func (env *ShellEnvironment) Getenv(key string) string {
 
 func (env *ShellEnvironment) Pwd() string {
 	defer env.trace(time.Now(), "Pwd")
+	env.lock.Lock()
 	defer func() {
+		env.lock.Unlock()
 		env.log(Debug, "Pwd", env.cwd)
 	}()
 	if env.cwd != "" {
@@ -312,8 +329,8 @@ func (env *ShellEnvironment) Pwd() string {
 		driveLetter := regex.GetCompiledRegex(`^[a-z]:`)
 		return driveLetter.ReplaceAllStringFunc(pwd, strings.ToUpper)
 	}
-	if env.args != nil && *env.args.PWD != "" {
-		env.cwd = correctPath(*env.args.PWD)
+	if env.CmdFlags != nil && env.CmdFlags.PWD != "" {
+		env.cwd = correctPath(env.CmdFlags.PWD)
 		return env.cwd
 	}
 	dir, err := os.Getwd()
@@ -334,7 +351,9 @@ func (env *ShellEnvironment) HasFiles(pattern string) bool {
 		env.log(Error, "HasFiles", err.Error())
 		return false
 	}
-	return len(matches) > 0
+	hasFiles := len(matches) > 0
+	env.debugF("HasFiles", func() string { return strconv.FormatBool(hasFiles) })
+	return hasFiles
 }
 
 func (env *ShellEnvironment) HasFilesInDir(dir, pattern string) bool {
@@ -345,7 +364,9 @@ func (env *ShellEnvironment) HasFilesInDir(dir, pattern string) bool {
 		env.log(Error, "HasFilesInDir", err.Error())
 		return false
 	}
-	return len(matches) > 0
+	hasFilesInDir := len(matches) > 0
+	env.debugF("HasFilesInDir", func() string { return strconv.FormatBool(hasFilesInDir) })
+	return hasFilesInDir
 }
 
 func (env *ShellEnvironment) HasFileInParentDirs(pattern string, depth uint) bool {
@@ -354,49 +375,59 @@ func (env *ShellEnvironment) HasFileInParentDirs(pattern string, depth uint) boo
 
 	for c := 0; c < int(depth); c++ {
 		if env.HasFilesInDir(currentFolder, pattern) {
+			env.log(Debug, "HasFileInParentDirs", "true")
 			return true
 		}
 
 		if dir := filepath.Dir(currentFolder); dir != currentFolder {
 			currentFolder = dir
 		} else {
+			env.log(Debug, "HasFileInParentDirs", "false")
 			return false
 		}
 	}
-
+	env.log(Debug, "HasFileInParentDirs", "false")
 	return false
 }
 
 func (env *ShellEnvironment) HasFolder(folder string) bool {
 	defer env.trace(time.Now(), "HasFolder", folder)
 	_, err := os.Stat(folder)
-	return !os.IsNotExist(err)
+	hasFolder := !os.IsNotExist(err)
+	env.debugF("HasFolder", func() string { return strconv.FormatBool(hasFolder) })
+	return hasFolder
 }
 
 func (env *ShellEnvironment) FileContent(file string) string {
 	defer env.trace(time.Now(), "FileContent", file)
+	if !filepath.IsAbs(file) {
+		file = filepath.Join(env.Pwd(), file)
+	}
 	content, err := ioutil.ReadFile(file)
 	if err != nil {
 		env.log(Error, "FileContent", err.Error())
 		return ""
 	}
-	return string(content)
+	fileContent := string(content)
+	env.log(Debug, "FileContent", fileContent)
+	return fileContent
 }
 
-func (env *ShellEnvironment) FolderList(path string) []string {
-	defer env.trace(time.Now(), "FolderList", path)
-	content, err := os.ReadDir(path)
+func (env *ShellEnvironment) LsDir(path string) []fs.DirEntry {
+	defer env.trace(time.Now(), "LsDir", path)
+	entries, err := os.ReadDir(path)
 	if err != nil {
-		env.log(Error, "FolderList", err.Error())
+		env.log(Error, "LsDir", err.Error())
 		return nil
 	}
-	var folderNames []string
-	for _, s := range content {
-		if s.IsDir() {
-			folderNames = append(folderNames, s.Name())
+	env.debugF("LsDir", func() string {
+		var entriesStr string
+		for _, entry := range entries {
+			entriesStr += entry.Name() + "\n"
 		}
-	}
-	return folderNames
+		return entriesStr
+	})
+	return entries
 }
 
 func (env *ShellEnvironment) PathSeparator() string {
@@ -410,6 +441,7 @@ func (env *ShellEnvironment) User() string {
 	if user == "" {
 		user = os.Getenv("USERNAME")
 	}
+	env.log(Debug, "User", user)
 	return user
 }
 
@@ -420,7 +452,9 @@ func (env *ShellEnvironment) Host() (string, error) {
 		env.log(Error, "Host", err.Error())
 		return "", err
 	}
-	return cleanHostName(hostName), nil
+	hostName = cleanHostName(hostName)
+	env.log(Debug, "Host", hostName)
+	return hostName, nil
 }
 
 func (env *ShellEnvironment) GOOS() string {
@@ -484,20 +518,20 @@ func (env *ShellEnvironment) HasCommand(command string) bool {
 
 func (env *ShellEnvironment) ErrorCode() int {
 	defer env.trace(time.Now(), "ErrorCode")
-	return *env.args.ErrorCode
+	return env.CmdFlags.ErrorCode
 }
 
 func (env *ShellEnvironment) ExecutionTime() float64 {
 	defer env.trace(time.Now(), "ExecutionTime")
-	if *env.args.ExecutionTime < 0 {
+	if env.CmdFlags.ExecutionTime < 0 {
 		return 0
 	}
-	return *env.args.ExecutionTime
+	return env.CmdFlags.ExecutionTime
 }
 
-func (env *ShellEnvironment) Args() *Args {
-	defer env.trace(time.Now(), "Args")
-	return env.args
+func (env *ShellEnvironment) Flags() *Flags {
+	defer env.trace(time.Now(), "Flags")
+	return env.CmdFlags
 }
 
 func (env *ShellEnvironment) BatteryInfo() ([]*battery.Battery, error) {
@@ -562,8 +596,8 @@ func (env *ShellEnvironment) BatteryInfo() ([]*battery.Battery, error) {
 
 func (env *ShellEnvironment) Shell() string {
 	defer env.trace(time.Now(), "Shell")
-	if *env.args.Shell != "" {
-		return *env.args.Shell
+	if env.CmdFlags.Shell != "" {
+		return env.CmdFlags.Shell
 	}
 	pid := os.Getppid()
 	p, _ := process.NewProcess(int32(pid))
@@ -581,8 +615,8 @@ func (env *ShellEnvironment) Shell() string {
 		return Unknown
 	}
 	// Cache the shell value to speed things up.
-	*env.args.Shell = strings.Trim(strings.Replace(name, ".exe", "", 1), " ")
-	return *env.args.Shell
+	env.CmdFlags.Shell = strings.Trim(strings.Replace(name, ".exe", "", 1), " ")
+	return env.CmdFlags.Shell
 }
 
 func (env *ShellEnvironment) HTTPRequest(targetURL string, timeout int, requestModifiers ...HTTPRequestModifier) ([]byte, error) {
@@ -607,6 +641,7 @@ func (env *ShellEnvironment) HTTPRequest(targetURL string, timeout int, requestM
 		env.log(Error, "HTTPRequest", err.Error())
 		return nil, err
 	}
+	env.log(Debug, "HTTPRequest", string(body))
 	return body, nil
 }
 
@@ -637,10 +672,10 @@ func (env *ShellEnvironment) HasParentFilePath(path string) (*FileInfo, error) {
 
 func (env *ShellEnvironment) StackCount() int {
 	defer env.trace(time.Now(), "StackCount")
-	if *env.args.StackCount < 0 {
+	if env.CmdFlags.StackCount < 0 {
 		return 0
 	}
-	return *env.args.StackCount
+	return env.CmdFlags.StackCount
 }
 
 func (env *ShellEnvironment) Cache() Cache {
@@ -695,9 +730,13 @@ func (env *ShellEnvironment) TemplateCache() *TemplateCache {
 	return tmplCache
 }
 
-func DirMatchesOneOf(env Environment, dir string, regexes []string) bool {
+func (env *ShellEnvironment) DirMatchesOneOf(dir string, regexes []string) bool {
+	return dirMatchesOneOf(dir, env.Home(), env.GOOS(), regexes)
+}
+
+func dirMatchesOneOf(dir, home, goos string, regexes []string) bool {
 	normalizedCwd := strings.ReplaceAll(dir, "\\", "/")
-	normalizedHomeDir := strings.ReplaceAll(env.Home(), "\\", "/")
+	normalizedHomeDir := strings.ReplaceAll(home, "\\", "/")
 
 	for _, element := range regexes {
 		normalizedElement := strings.ReplaceAll(element, "\\\\", "/")
@@ -705,7 +744,6 @@ func DirMatchesOneOf(env Environment, dir string, regexes []string) bool {
 			normalizedElement = strings.Replace(normalizedElement, "~", normalizedHomeDir, 1)
 		}
 		pattern := fmt.Sprintf("^%s$", normalizedElement)
-		goos := env.GOOS()
 		if goos == WindowsPlatform || goos == DarwinPlatform {
 			pattern = "(?i)" + pattern
 		}
