@@ -190,6 +190,15 @@ func (g *Git) shouldDisplay() bool {
 			return true
 		}
 
+		// check for separate git folder(--separate-git-dir)
+		// check if the folder contains a HEAD file
+		if g.env.HasFilesInDir(g.gitWorkingFolder, "HEAD") {
+			gitFolder := strings.TrimSuffix(g.gitRootFolder, ".git")
+			g.gitRootFolder = g.gitWorkingFolder
+			g.gitWorkingFolder = gitFolder
+			g.gitRealFolder = gitFolder
+			return true
+		}
 		return false
 	}
 	return false
@@ -256,6 +265,8 @@ func (g *Git) setGitStatus() {
 		UPSTREAM     = "# branch.upstream "
 		BRANCHSTATUS = "# branch.ab "
 	)
+	// firstly assume that upstream is gone
+	g.UpstreamGone = true
 	g.Working = &GitStatus{}
 	g.Staging = &GitStatus{}
 	output := g.getGitCommandOutput("status", "-unormal", "--branch", "--porcelain=2")
@@ -484,8 +495,14 @@ func (g *Git) getWorktreeContext() int {
 	if !g.env.HasFolder(g.gitRootFolder + "/worktrees") {
 		return 0
 	}
-	worktreeFolders := g.env.FolderList(g.gitRootFolder + "/worktrees")
-	return len(worktreeFolders)
+	worktreeFolders := g.env.LsDir(g.gitRootFolder + "/worktrees")
+	var count int
+	for _, folder := range worktreeFolders {
+		if folder.IsDir() {
+			count++
+		}
+	}
+	return count
 }
 
 func (g *Git) getOriginURL(upstream string) string {
