@@ -5,7 +5,11 @@ import (
 	link "net/url"
 )
 
+// url builds an hyperlink if url is not empty, otherwise returns the text only
 func url(text, url string) (string, error) {
+	if url == "" {
+		return text, nil
+	}
 	_, err := link.ParseRequestURI(url)
 	if err != nil {
 		return "", err
